/* fed_monit v6.62618e-34 J-s */

#include <stdio.h>
#include <unistd.h>
#include <syslog.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <time.h>
#include <sys/wait.h>
#include <sys/signal.h>

#ifdef __FreeBSD__
#include <machine/cpufunc.h>
#else
#include <asm/io.h>		/* GNU/Linux */
#endif

#define BASE_ADDRESS 	0x378	               /* 0x3BC, 0x378, 0x278 */
#define LOGS 		"/usr/local/fed_monit/logs"	    /* audio log dir */
#define BREC	        "/usr/bin/brec"	  /* path to audio recording program */
#define HZ		"44100"	                         /* makes good MP3's */
#define BITS		"16"                             /*      ""          */
#define SEC		"20"	               /* seconds of audio to record */

#define NORMAL 		"\033[0m"
#define BOLDWHITE 	"\033[37m\033[1m"

unsigned char Nib1, Nib2;
char *freq, *optarg, buf[64];
char *sec = SEC;
char *brec = BREC;
char *bits = BITS;
char *hz = HZ;
int a, c, on, status;
pid_t pid;
extern int errno;
struct tm *l;
time_t t;

int check (void);
int initial (void);
void usage (char *);
void onalarm (int);

int
main (int argc, char **argv)
{

  if (geteuid () && getuid ())
    {
      fprintf (stderr, "\n\n%s must be run as root to access hardware ports.\n", argv[0]);
      fprintf (stderr, "Leaving...\n\n");
      exit (1);
    }

  if (argc < 3)
      usage (argv[0]);

  while ((a = getopt (argc, argv, "of:")) != EOF)
    {
      switch (a)
	{
	case 'o':		/* Scanner is on */
	  on = 1;
	  break;
	case 'f':		/* Frequency to log */
	  freq = optarg;
	  break;
	default:
	  usage (argv[0]);
	  break;
	}
    }

  if (!freq)
    {
      usage (argv[0]);
    }

  if (on != 1)
    {
      fprintf (stderr, "\n\t%sScanner must be already turned ON.%s\n", BOLDWHITE, NORMAL);
      fprintf (stderr, "\tIf it is not, turn it on now.\n\n");
      fprintf (stderr, "Hit any key to continue...");
      getchar ();
    }

  initial ();

  for (;;)
    {

      check ();

      if (Nib2 == 0)
	{
	  c++;
	  t = time (NULL);
	  l = localtime (&t);

#ifdef DEBUG
	  printf ("Nib1 = 0x%x - Fake 20 second recording...\n\a", Nib1);
	  sleep (20);
#else
	  if (chdir (LOGS) != 0)
	    {
	      perror ("chdir");
	      exit (errno);
	    }

	  snprintf (buf, sizeof (buf), "%s_%.2d.%.2d.%.4d-%.2d:%.2d:%.2d.wav", freq, (1 + l->tm_mon), (l->tm_mday), (1900 + l->tm_year), (l->tm_hour), (l->tm_min), (l->tm_sec));

	  printf ("(%d) Activity monitored at %s MHz on %s\n\a", c, freq, asctime (l));
	  syslog (LOG_NOTICE, "(%d) Activity monitored at %s MHz", c, freq);

	  if ((pid = fork ()) < 0)
	    perror ("fork");
	  else if (pid == 0)
	    {
	      if (execl (brec, brec, "-t", sec, "-s", hz, "-b", bits, "-w", buf, (char *) 0) < 0)
		perror ("execl");
	    }

	  signal (SIGALRM, onalarm);

	  alarm ((atoi (SEC) + 5));

	  if (wait (&status) == -1 || (status & 0177) != 0)
	    fprintf (stderr, "%s killed.\n", BREC);

#endif
	  printf ("Recording done.\n\n");
	}			/* Loop forever, lots of cpu time */
    }
}

int
check (void)
{

  Nib2 = (inb (BASE_ADDRESS + 1) & 0x40); /* Isolate Pin 10 - 01000000 */

#ifdef DEBUG
  printf ("Nib2 = 0x%x\n", Nib2);
#endif
  return (0);
}

int
initial (void)
{

  printf ("Running...\n\n");

#ifdef __FreeBSD__
  if (open ("/dev/io", O_RDWR) < 0)
#else
  if (ioperm (BASE_ADDRESS, 3, 1) < 0)
#endif
    {
      fprintf (stderr, "\nERROR: Can't open port 0x%x\n", BASE_ADDRESS);
      perror ("port");
      exit (errno);
    }

  Nib1 = (inb (BASE_ADDRESS + 1) & 0x40); /* Isolate Pin 10 - 01000000 */ 

#ifdef DEBUG
  printf ("Nib1 = 0x%x\n", Nib1);
#endif

  return (0);
}

void
usage (char *pname)
{
  fprintf (stderr, "%s ", pname);
  fprintf (stderr, "[-o] -f <frequency>\n\n\t-o If scanner is already on\n\t");
  fprintf (stderr, "-f frequency to record\n\n");
  exit (1);
}

void
onalarm (int val)
{
  kill (pid, SIGHUP);
  kill (pid + 1, SIGHUP);
}
